﻿#If NoCompile Then
the compact-format is a primitive file-format, similar to CSV, but can store multiple tables in one file. There are 3 special-chars, reserved for structurating data: 
* _Sep: 0xf018 - at the beginning of each data-entry. 
* DBNull: 0xf019 - at the beginning of each table. Furthermore DBNull is public constant, and can be used to indicate a null-entry (and distinguish it from an empty string). 
* _NewLineMask: 0xf017 - maskes newlines within data-entries.

this is the compact-format-syntax:
a line, starting with DBNull is a tableheader. text in tableheaders will be ignored, but may improve readability
after that, lines, starting with _Sep, are dataRows.
each data-entry starts with _Sep (and the leading _Sep of a datarow belongs to its first Entry)
lines, starting with anything else than _Sep, will be ignored.
An empty line indicates the end of the table.
After a table, any line will be ignored, until the next tableheader.
#End If 'NoCompile

Imports System.Text
Namespace System.IO

   Public Class CompactReadWrite : Implements IDisposable

      Public Const DBNull As String = Microsoft.VisualBasic.ChrW(&HF019)
      Private Const _Sep As Char = Microsoft.VisualBasic.ChrW(&HF018)
      Private Const _NewLineMask As Char = Microsoft.VisualBasic.ChrW(&HF017)
      Private Shared _Illegal As Char() = New Char() {_Sep, _NewLineMask}

      Private _LeaveOpen As Boolean, _Reader As TextReader, _Writer As TextWriter

      Public Sub New( Writer As TextWriter, Optional  LeaveWriterOpen As Boolean = False)
         _LeaveOpen = LeaveWriterOpen
         _Writer = Writer
      End Sub

      Public Sub New( Reader As TextReader, Optional  LeaveReaderOpen As Boolean = False)
         _LeaveOpen = LeaveReaderOpen
         _Reader = Reader
      End Sub

      Public Sub WriteHeader(Optional  tableName As String = "table", Optional  columnNames As IEnumerable(Of String) = Nothing)
         If columnNames.NotNull() Then
            tableName += ": " & JoinData(", ", columnNames)
         End If
         _Writer.WriteLine(DBNull & tableName)
      End Sub
      Public Sub WriteRow( data As IEnumerable(Of String))
         _Writer.WriteLine(_Sep & JoinData(_Sep, data))
      End Sub
      Private Function JoinData( separator As String,  data As IEnumerable(Of String)) As String
         Dim sb = New StringBuilder()
         For Each s In data
            If s.IndexOfAny(_Illegal) >= 0 Then
               Throw Me.Exception("illegal chars in data")
            End If
            sb.Append(separator).Append(s.Replace(Environment.NewLine, _NewLineMask))
         Next
         Return sb.ToString(separator.Length, sb.Length - separator.Length)
      End Function

      Public Sub CloseWriteTable()
         _Writer.WriteLine()
      End Sub

      Protected Overridable Sub Dispose( disposing As Boolean)
         If Not disposing OrElse _LeaveOpen Then Return
         If _Writer.NotNull Then
            _Writer.Dispose()
            _Writer = Nothing
         ElseIf _Reader.NotNull Then
            _Reader.Dispose()
            _Reader = Nothing
         End If
      End Sub

      Public Sub Dispose() Implements IDisposable.Dispose
         Dispose(True)
         GC.SuppressFinalize(Me)
      End Sub

      Public Iterator Function ReadTable() As IEnumerable(Of String())
         Dim s As String
         Do 'wind forward to DBNull, which indicates Begin of next Table
            s = _Reader.ReadLine
            If s.Null Then Return ' end of File
         Loop Until s.StartsWith(DBNull)
         Do
            Do 'wind forward to _Sep, which indicates Begin of next TableRow
               s = _Reader.ReadLine
               If s.Length = 0 Then Return 'end of Table
            Loop Until s(0) = _Sep
            Yield s.Replace(_NewLineMask, Environment.NewLine).Substring(1).Split(_Sep)
         Loop
      End Function

   End Class
End Namespace
