﻿Imports System.Runtime.CompilerServices

Namespace System.Windows.Forms

   Public Module ControlX
#Region "DataGridView"
      ''' <summary> enumeriert die DataRowViews, bei denen die CheckboxCell gecheckt ist </summary>
      <Extension()> _
      Public Iterator Function CheckedDrvs(col As DataGridViewCheckBoxColumn) As IEnumerable(Of DataRowView)
         Dim dgv = col.DataGridView
         Dim colIndx = col.Index
         For i = 0 To dgv.RowCount - 1 - dgv.AllowUserToAddRows.AsIndex
            Dim rwDgv = dgv.Rows(i)
            If CBool(rwDgv.Cells(colIndx).EditedFormattedValue) Then Yield DirectCast(rwDgv.DataBoundItem, DataRowView)
         Next
      End Function
      ''' <summary> enumeriert die DataRows, bei denen die CheckboxCell gecheckt ist </summary>
      <Extension()> _
      Public Function CheckedDrvs(Of T As DataRow)(col As DataGridViewCheckBoxColumn) As IEnumerable(Of T)
         Return From drv In col.CheckedDrvs Select DirectCast(drv.Row, T)
      End Function
      <Extension()> _
      Public Sub SetValue(col As DataGridViewColumn, rowIndex As Integer, value As Object)
         col.DataGridView(col.Index, rowIndex).Value = value
      End Sub
      ''' <summary> returns null if invalid rowIndex </summary>
      <Extension()> _
      Public Function DataRow(Of T As DataRow)(grd As DataGridView, rowIndex As Integer) As T
         If rowIndex < 0 Then Return Nothing
         Return grd.Rows(rowIndex).DataRow(Of T)()
      End Function
      <Extension()> _
      Public Function DataRow(Of T As DataRow)(dgvRow As DataGridViewRow) As T
         Dim itm = DirectCast(dgvRow.DataBoundItem, DataRowView)
         Return If(itm.Null, Nothing, DirectCast(itm.Row, T))
      End Function
      ''' <summary> returns null if invalid rowIndex </summary>
      <Extension()> _
      Public Function DataRow(grd As DataGridView, rowIndex As Integer) As DataRow
         If rowIndex < 0 Then Return Nothing
         Dim itm = DirectCast(grd.Rows(rowIndex).DataBoundItem, DataRowView)
         Return If(itm.Null, Nothing, itm.Row)
      End Function

      <Extension()> _
      Public Function IsColumnBound(grd As DataGridView, columnIndex As Integer, dataColumn As DataColumn) As Boolean
         If columnIndex < 0 Then Return False
         Return grd.Columns(columnIndex).DataPropertyName = dataColumn.ColumnName
      End Function
      <Extension()> _
      Public Function Column(grd As DataGridView, dataColumn As DataColumn) As DataGridViewColumn
         Return grd.ColumnsX(dataColumn).First
      End Function
      <Extension()> _
      Public Function ColumnsX(grd As DataGridView, dataColumn As DataColumn) As IEnumerable(Of DataGridViewColumn)
         Return grd.Columns.Cast(Of DataGridViewColumn).Where(Function(col) col.DataPropertyName = dataColumn.ColumnName)
      End Function
#End Region 'DataGridView

#Region "TabControl"
      ''' <summary>
      ''' Dieser Schwachsinn im Form_Load ausgeführt behebt einen Bug beim DataBinding auf beim Start nicht sichtbaren Tabpages.
      ''' </summary>
      <Extension()> _
      Public Sub InitBindability(tc As TabControl)
         Dim parent = tc.FindForm
         parent.SuspendLayout()
         For Each tp As TabPage In tc.TabPages
            tp.SuspendLayout()
            For Each ctl In tp.Controls.Cast(Of Control).ToArray
               parent.Controls.Add(ctl)
               tp.Controls.Add(ctl)
            Next
            tp.ResumeLayout()
         Next
         parent.ResumeLayout()
      End Sub

      ''' <summary>hides Riders of TabControl</summary>
      <Extension()> _
      Public Sub HideRiders(tc As TabControl)
         tc.SuspendLayout()
         tc.SizeMode = TabSizeMode.Fixed
         tc.ItemSize = New Size(0, 1)
         tc.Appearance = TabAppearance.Buttons
         tc.ResumeLayout()
      End Sub

      ''' <summary>Setzt den Focus nachm Tab-Selectieren wieder zurück wo er herkam</summary>
      <Extension()> _
      Public Sub ToFront(tp As TabPage)
         With DirectCast(tp.Parent, TabControl)
            If .SelectedTab Is tp Then Return
            Dim focused = .FindForm.ActiveControl
            .SelectedTab = tp
            If focused.NotNull Then BusyDelay.SetCallback(AddressOf focused.Focus)
         End With
      End Sub
#End Region 'TabControl

   End Module

End Namespace
